local Pendulum = LibStub("AceAddon-3.0"):NewAddon("Pendulum", "AceTimer-3.0")

local frame1, frame2, frame3
local frame = {}
local layers = {}
local size, db
local sec, mn, hr
local lastsec, lastmn, lasthr = -1,-1, -1
local skins = {}
local skinlist = {}

local math_rad = math.rad
local math_sin = math.sin
local math_cos = math.cos

local function rotate(texture, angle, scale)
	scale = scale or 1
	local a_cos = math_cos(angle)
	local a_sin = math_sin(angle)
	local CS = (a_cos + a_sin + 1) * 0.5 * scale
	local cS = (a_sin - a_cos + 1) * 0.5 * scale
	local cs = (-a_sin - a_cos + 1) * 0.5 * scale
	local Cs = (a_cos - a_sin + 1) * 0.5 * scale
	
	texture:SetTexCoord(cs, Cs, cS, cs, Cs, CS, CS, cS)
end

local defaults = {
	profile = {
		facecolor = { r = 1, g = 1, b = 1, a = 1 },
		secondcolor = { r = 1, g = 1, b = 1, a = 1 },
		minutecolor = { r = 1, g = 1, b = 1, a = 1 },
		hourcolor = { r = 1, g = 1, b = 1, a = 1 },
		size = 60,
	}
}

local function giveOptions()
	local options = {
		type = "group",
		name = "Pendulum",
		args = {
			radius = {
				type = "range",
				name = "Size",
				desc = "Clock size",
				min = 10, max = 240, step = 1,
				get = function( k ) return db.profile.size end,
				set = function( k, v )
					db.profile.size = v
					frame1:SetWidth(v)
					frame1:SetHeight(v)
					Pendulum:ApplySkin()
				end,
				order = 1,
				width = "full",
			},
			skin = {
				type = 'select',
				values = skinlist,
				name = L["Skin"],
				get = function() return db.profile.curskin or "Pendulum12Hour" end,
				set = function(info,c) db.profile.curskin = c Pendulum:ApplySkin() end,
				style = "dropdown",
				order = 2,
				width = "full",
			},
		},
	}
	return options
end

function Pendulum:OnInitialize()
	db = LibStub("AceDB-3.0"):New("PendulumDB", defaults, "Default")
	
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("Pendulum", giveOptions)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Pendulum", "Pendulum")
end

function Pendulum:OnEnable()
	self:CreateClock()
	self:ScheduleRepeatingTimer("UpdateClock", .1)
end

function Pendulum:CreateClock()
	if frame1 then return end

	frame1 = CreateFrame("Button", "PendulumFrame1", UIParent)
	
	local size = db.profile.size
	frame1:SetWidth(size)
	frame1:SetHeight(size)
	frame1:ClearAllPoints()
	frame1:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	frame1:SetFrameLevel(1)

	frame1:EnableMouse(true)
	frame1:RegisterForDrag("LeftButton")
	frame1:RegisterForClicks("AnyUp")
	frame1:SetMovable(true)
	frame1:SetScript("OnDragStart", function() if IsAltKeyDown() then this:StartMoving() end end)
	frame1:SetScript("OnDragStop", function() this:StopMovingOrSizing() self:SavePosition() end)

	frame2 = CreateFrame("Frame", "PendulumFrame2", UIParent)
	
	frame2:ClearAllPoints()
	frame2:SetAllPoints(frame1)
	frame2:SetFrameLevel(2)

	frame3 = CreateFrame("Frame", "PendulumFrame3", UIParent)
	
	frame3:ClearAllPoints()
	frame3:SetAllPoints(frame1)
	frame3:SetFrameLevel(3)
	
	frame[1] = frame1
	frame[2] = frame2
	frame[3] = frame3

	self:ApplySkin()
	
	self:RestorePosition()
end

function Pendulum:AddSkin(name,label,skin)
	skins[name] = skin
	skinlist[name] = label
end

function Pendulum:ApplySkin()
	curskin = skins[db.profile.curskin or "Pendulum12Hour"] or skins["Pendulum12Hour"]
	local size = db.profile.size / 60
	for i = 1, #curskin do
		local skinlayer = curskin[i]
		local layer = layers[i] or frame1:CreateTexture()
		layer:SetParent(frame[skinlayer.FrameLevel or 1])
		layer:SetDrawLayer(skinlayer.DrawLayer)
		layer:SetTexture(skinlayer.Texture)
		layer:SetVertexColor(1,1,1,1)
		layer:SetWidth(skinlayer.Width * size)
		layer:SetHeight(skinlayer.Height * size)
		layer:ClearAllPoints()
		layer:SetPoint("CENTER",frame1,"CENTER",(skinlayer.XOffset or 0) * size,(skinlayer.YOffset or 0) * size)
		if skinlayer.Rate then
			layer.dynamic = true
			layer.rate = skinlayer.Rate
		else
			layer.dynamic = nil
			layer:SetTexCoord(0,1,0,1)
		end
		layer:Show()
		layers[i] = layer
	end
	for i = #curskin + 1, #layers do
		layers[i]:Hide()
	end
end

local div60 = 1/60
function Pendulum:UpdateClock()
	if not curskin then return end
	sec = mod( time(), 60 )
	if sec ~= lastsec then
		lastsec = sec
		mn = tonumber(date("%M")) * 60 + sec
		if curskin.Is24Hour then
			hr = tonumber(date("%H")) * 3600 + mn
		else
			hr = tonumber(date("%I")) * 3600 + mn
		end
		for i = 1, #layers do
			local layer = layers[i]
			if layer.dynamic and layer:IsVisible() then
				Pendulum:UpdateLayer(layer,hr)
			end
		end
	end
end

function Pendulum:UpdateLayer(layer,t)
	local angle = math_rad(180 + t * layer.rate)
	rotate(layer, angle)
end

function Pendulum:SavePosition()
	local s = frame1:GetEffectiveScale()

	local x, y = frame1:GetCenter()
	
	db.profile.posx = x * s
	db.profile.posy = y * s
end

function Pendulum:RestorePosition()
	local x = db.profile.posx
	local y = db.profile.posy

	if not x or not y then return end

	local s = frame1:GetEffectiveScale()

	frame1:ClearAllPoints()
	frame1:SetPoint("CENTER", UIParent, "BOTTOMLEFT", x / s, y / s)
end
